from django.contrib.auth.backends import ModelBackend
from django.contrib.auth.models import User


class CustomUserModelBackend(ModelBackend):
    def authenticate(self, username=None, password=None):
        """Allow users to log in with their email as well as username."""
        kw = 'email__iexact' if '@' in username else 'username'
        kwargs = {kw: username}
        try:
            user = User.objects.get(**kwargs)
        except User.DoesNotExist:
            pass
        else:
            if user.check_password(password):
                return user



from django import forms
from django.contrib.auth.forms import AuthenticationForm
from django.utils.translation import ugettext_lazy as _


class IncunaAuthenticationForm(AuthenticationForm):
    username = forms.CharField(label=_('Email'), max_length=320)



import re

from django.conf import settings
from django.contrib import messages
from django.http import HttpResponseRedirect


EXEMPT_URLS = [re.compile('^%s$' % settings.LOGIN_URL.lstrip('/')), re.compile('^%s$' % settings.LOGOUT_URL.lstrip('/'))]
EXEMPT_URLS += [re.compile(expr) for expr in getattr(settings, 'LOGIN_EXEMPT_URLS', [])]
PROTECTED_URLS = [re.compile(expr) for expr in getattr(settings, 'LOGIN_PROTECTED_URLS', [r'^'])]


class LoginRequiredMiddleware:
    '''
    Middleware that requires a user to be authenticated to view any page in
    LOGIN_PROTECTED_URLS other than LOGIN_URL. Exemptions to this requirement
    can optionally be specified in settings via a list of regular expressions
    in LOGIN_EXEMPT_URLS (which you can copy from your urls.py).

    Requires authentication middleware and template context processors to be
    loaded. You'll get an error if they aren't.

    Will default to protecting everything if LOGIN_PROTECTED_URLS is not in
    settings.


    Original code from:
    http://onecreativeblog.com/post/59051248/django-login-required-middleware

    This version has been modified to allow us to define areas of the site to
    password protect instead of protecting everything under /.
    '''
    def process_request(self, request):
        assert hasattr(request, 'user'), ("The Login Required middleware "
            "requires authentication middleware to be installed. Edit your "
            "MIDDLEWARE_CLASSES setting to insert "
            "django.contrib.auth.middlware.AuthenticationMiddleware"
            "If that doesn't work, ensure your TEMPLATE_CONTEXT_PROCESSORS "
            "setting includes 'django.core.context_processors.auth'.")

        # Jump over this middleware if not a protected url.
        path = request.path_info.lstrip('/')
        path_is_exempt = any(m.match(path) for m in EXEMPT_URLS)
        path_is_protected = any(m.match(path) for m in PROTECTED_URLS)
        if path_is_exempt or not path_is_protected:
            return

        # Jump over this middleware if user logged in.
        if request.user.is_authenticated():
            return

        # Add a message, and redirect to login.
        messages.info(request, 'You must be logged in to view this page.')
        return HttpResponseRedirect(settings.LOGIN_URL + '?next=' + request.path_info)




from django.conf import settings
from django.conf.urls.defaults import patterns, url
from django.core.urlresolvers import get_callable
from django.views.generic import RedirectView

try:
    from django.core.urlresolvers import reverse_lazy
except ImportError:
    # Support for django <1.4
    from django.core.urlresolvers import reverse
    from django.utils.functional import lazy
    reverse_lazy = lazy(reverse, str)


auth_form = get_callable(getattr(settings, 'INCUNA_AUTH_LOGIN_FORM', 'incuna_auth.forms.IncunaAuthenticationForm'))


urlpatterns = patterns('django.contrib.auth.views',
    url(r'^login/$', 'login', {'authentication_form': auth_form}, name='auth_login'),
    url(r'^logout/$', 'logout', {'template_name': 'registration/logout.html'}, name='auth_logout'),
    # Change password (when logged in)
    url(r'^password/change/$', 'password_change', name='auth_password_change'),
    url(r'^password/change/done/$', 'password_change_done', name='password_change_done'),
    # Reset password (when not logged in via unique email link)
    url(r'^password/reset/$', 'password_reset', name='auth_password_reset'),
    url(r'^password/reset/done/$', 'password_reset_done', name='auth_password_reset_done'),
    url(r'^password/reset/confirm/(?P<uidb36>[0-9A-Za-z]+)-(?P<token>.+)/$', 'password_reset_confirm', name='auth_password_reset_confirm'),
    url(r'^password/reset/complete/$', 'password_reset_complete', name='auth_password_reset_complete'),
    url(r'^sso/$', RedirectView.as_view(url=reverse_lazy('admin:admin_sso_openiduser_start')), name='sso_login'),
)



from setuptools import find_packages, setup


install_requires = ('django-admin-sso',)

setup(
    name='incuna-auth',
    version='0.6.4',
    url='http://github.com/incuna/incuna-auth',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    description='Provides authentication parts.',
    long_description=open('README.rst').read(),
    author='Incuna Ltd',
    author_email='admin@incuna.com',
)
